==> diablo-1.00.tgz: diablo/lib/buffer.c <==

/*
 * LIB/BUFFER.C
 *
 * (c)Copyright 1997, Matthew Dillon, All Rights Reserved.  Refer to
 *    the COPYRIGHT file in the base directory of this distribution 
 *    for specific rights granted.
 *
 */

#include "defs.h"

Prototype char *bgets(Buffer **pb, int fd);
Prototype void bclose(Buffer **pb);

char *
bgets(Buffer **pb, int fd)
{
    Buffer *b;
    int i;

    if ((b = *pb) == NULL) {
	b = *pb = malloc(sizeof(Buffer));
	b->bu_Beg = 0;
	b->bu_NLScan = 0;
	b->bu_End = 0;
    }
    for (;;) {
	/*
	 * look for a newline, return the line as appropriate
	 */

	for (i = b->bu_NLScan; i < b->bu_End; ++i) {
	    if (b->bu_Data[i] == '\n') {
		char *p = b->bu_Data + b->bu_Beg;
		b->bu_Data[i] = 0;
		b->bu_Beg = i + 1;
		b->bu_NLScan = b->bu_Beg;
		if (b->bu_Beg == b->bu_End) {
		    b->bu_Beg = 0;
		    b->bu_NLScan = 0;
		    b->bu_End = 0;
		}
		return(p);
	    }
	}

	/*
	 * If there is no room to append new data, attempt to
	 * make some.
	 */
	if (b->bu_End == sizeof(b->bu_Data) && b->bu_Beg != 0) {
	    bcopy(b->bu_Data + b->bu_Beg, b->bu_Data, b->bu_End - b->bu_Beg);
	    b->bu_End -= b->bu_Beg;
	    b->bu_NLScan -= b->bu_Beg;
	    b->bu_Beg = 0;
	}

	/*
	 * If the buffer is full, we have an overflow problem
	 */
	if (b->bu_End == sizeof(b->bu_Data)) {
	    syslog(LOG_ERR, "Line buffer overflow pipe fd %d", fd);
	    return((void *)-1);
	}
	/*
	 * otherwise read some data and repeat
	 */
	{
	    int n;

	    n = read(fd, b->bu_Data + b->bu_End, sizeof(b->bu_Data)-b->bu_End);

	    /*
	     * EOF
	     */
	    if (n == 0) {
		return((void *)-1);
	    }
	    if (n < 0) {
		if (errno == EAGAIN || errno == EINTR)
		    return(NULL);
		return((void *)-1);
	    }
	    b->bu_End += n;
	}
    }
    /* not reached */
    return((void *)-1);
}

void
bclose(Buffer **pb)
{
    Buffer *b;

    if ((b = *pb) != NULL) {
	free(b);
	*pb = NULL;
    }
}

==> diablo-1.05.tgz: diablo/lib/buffer.c <==

/*
 * LIB/BUFFER.C
 *
 * (c)Copyright 1997, Matthew Dillon, All Rights Reserved.  Refer to
 *    the COPYRIGHT file in the base directory of this distribution 
 *    for specific rights granted.
 *
 */

#include "defs.h"

Prototype char *bgets(Buffer **pb, int fd);
Prototype void bclose(Buffer **pb);

char *
bgets(Buffer **pb, int fd)
{
    Buffer *b;
    int i;

    if ((b = *pb) == NULL) {
	int pgSize;

	b = *pb = pagealloc(&pgSize);
	b->bu_Beg = 0;
	b->bu_NLScan = 0;
	b->bu_End = 0;
	b->bu_Max = pgSize - offsetof(Buffer, bu_Data[0]);
    }
    for (;;) {
	/*
	 * look for a newline, return the line as appropriate
	 */

	for (i = b->bu_NLScan; i < b->bu_End; ++i) {
	    if (b->bu_Data[i] == '\n') {
		char *p = b->bu_Data + b->bu_Beg;
		b->bu_Data[i] = 0;
		b->bu_Beg = i + 1;
		b->bu_NLScan = b->bu_Beg;
		if (b->bu_Beg == b->bu_End) {
		    b->bu_Beg = 0;
		    b->bu_NLScan = 0;
		    b->bu_End = 0;
		}
		return(p);
	    }
	}

	/*
	 * If there is no room to append new data, attempt to
	 * make some.
	 */
	if (b->bu_End == b->bu_Max && b->bu_Beg != 0) {
	    memmove(b->bu_Data, b->bu_Data + b->bu_Beg, b->bu_End - b->bu_Beg);
	    b->bu_End -= b->bu_Beg;
	    b->bu_NLScan -= b->bu_Beg;
	    b->bu_Beg = 0;
	}

	/*
	 * If the buffer is full, we have an overflow problem
	 */
	if (b->bu_End == b->bu_Max) {
	    syslog(LOG_ERR, "Line buffer overflow pipe fd %d", fd);
	    return((void *)-1);
	}
	/*
	 * otherwise read some data and repeat
	 */
	{
	    int n;

	    n = read(fd, b->bu_Data + b->bu_End, b->bu_Max - b->bu_End);

	    /*
	     * EOF
	     */
	    if (n == 0) {
		return((void *)-1);
	    }
	    if (n < 0) {
		if (errno == EAGAIN || errno == EINTR)
		    return(NULL);
		return((void *)-1);
	    }
	    b->bu_End += n;
	}
    }
    /* not reached */
    return((void *)-1);
}

void
bclose(Buffer **pb)
{
    Buffer *b;

    if ((b = *pb) != NULL) {
	pagefree(b);
	*pb = NULL;
    }
}
